require 'singleton'
require 'forwardable'
require 'benchmark'

require 'benchmark_harness/stats'
require 'benchmark_harness/collection'

class BenchmarkHarness
  include Singleton
  
  class << self
    # Cheap way to easily treat class like it's an instance, because why not
    extend Forwardable
    def_delegators :instance, :collections, :measure, :flush!, :wrap_method
  end
  
  def collections
    @collections ||= Hash.new{|h,k| h[k] = Collection.new}
  end
  
  def flush!
    @collections = nil
  end
  
  def measure(collection_name)
    result = nil
    m = Benchmark.measure{
      result = yield
    }
    collections[collection_name] << m.to_a[1..-1]
    result
  end
  
  # @param [String] signature
  # @example "BenchmarkHarness#measure" instance method
  # @example "BenchmarkHarness.measure" class method
  def wrap_method(signature, collection_name)
    if signature =~ /#/
      split_on = "#"
      class_method = false
    elsif signature =~ /\./
      split_on = "."
      class_method = true
    else
      raise ArgumentError, "your signature must contain a method"
    end
        
    constant_name, method_name = signature.split(split_on)
    method_name_safe = method_name.gsub(/[^\w\d]/, '') # strip out ie !
    constant = get_constant(constant_name)
    
    method_definition = <<-END
      alias :#{method_name_safe}_pre_benchmark_harness :#{method_name}
      def #{method_name}(*args, &block)
        BenchmarkHarness.measure(:#{collection_name}){result = #{method_name_safe}_pre_benchmark_harness(*args, &block)}
      end
    END
    
    if class_method
      constant = class << constant; self; end
    end
    
    constant.class_eval method_definition
  end
  
  # @param [String] name
  def get_constant(name)
    pieces = name.split("::")
    parent_constant = Object
    pieces.each do |piece|
      if parent_constant.const_defined?(piece)
        parent_constant = parent_constant.const_get(piece)
      else
        raise NameError, "#{name} contains an undefined constant; it may be autoloaded"
      end
    end
    parent_constant
  end
end

class Array
  include BenchmarkHarness::Stats
end