class BenchmarkHarness
  class Collection < Array
    
    # order corresponds to values in each sample
    KEYS        = [:user, :system, :children_user, :children_system, :real]
    STATS       = [:mean, :median, :mode, :range, :min, :max, :sum, :standard_deviation, :variance]
    STATS_HUMAN = STATS.collect{|s| s.to_s.sub(/_/, " ").capitalize}
    
    # Sorts values into named vectors:
    #
    # @example {:user => [1,3,2,3], :system => [0,0,0,1]}
    def reconciled
      return @reconciled if @reconciled
      @reconciled = Hash.new{|h,k| h[k] = []}
      self.each do |sample|
        KEYS.each_with_index do |key, index|
          @reconciled[key] << sample[index]
        end
      end
      @reconciled
    end
    
    # Specify that the corresponding easystats method can be used
    # also defines the method so that it can optionally take a key.
    # If they key is present, return result for just that vector;
    # otherwise return hash of results for all keys/vectors    
    STATS.each do |method_name|
      class_eval <<-END
        def #{method_name}(key = nil)
          if key
            reconciled[key].#{method_name}
          else
            result = {}
            KEYS.each do |key|
              result[key] = reconciled[key].#{method_name}
            end
            result
          end
        end
      END
    end
    
    # Formatted output
    def print
      label_size = STATS_HUMAN.max{|a, b| a.size <=> b.size}.size
      puts sprintf("%#{label_size}s %d", "Samples:", self.size)
      if self.size > 0
        puts sprintf("%#{label_size}s: %10s %10s %10s %10s %10s", "Displaying", "user", "system", "ch.user", "ch.system", "real")
        STATS_HUMAN.each_with_index do |label, i|
          puts sprintf("%#{label_size}s: %10.4g %10.4g %10.4g %10.4g %10.4g", label, *KEYS.collect{|k| self.send(STATS[i], k)})
        end
      end
    end
  end
end